from pydantic import BaseModel, validator
from ..request.cep import Cep
from typing import Optional


class Endereco(BaseModel):
    rua: str
    numero: str
    bairro: str
    cidade: str
    estado: str
    cep: str
    complemento: Optional[str] = None
    referencia: Optional[str] = None

    @validator("cep")
    def validate_cep(cls, value):
        response = Cep(version=1).get(value)
        return response.cep



from pydantic import BaseModel, validator
from typing import Optional
from ..validate import Email


class Contato(BaseModel):
    telefone: Optional[str]
    email: Optional[str]

    @validator("email")
    def validate_email(cls, value):
        Email().validate(value)
        return value



from pydantic import BaseModel, validator
from .endereco import Endereco
from .contato import Contato
from ..validate import Cnpj


class EmpresaBase:
    cnpj: str
    razao_social: str
    nome_fantasia: str
    endereco: Endereco
    contato: Contato

    @validator("cnpj")
    def validate_cnpj(cls, value):
        cnpj = Cnpj()
        cnpj.validate(value)
        return cnpj.mask(value)


class Empresa(EmpresaBase, BaseModel):
    ...


class EmpresaContatoOptional(EmpresaBase, BaseModel):
    contato: Contato = None



from flask import (
    Flask,
    render_template,
    request,
    render_template_string,
    session,
    redirect,
    url_for,
)
import os
from flask.blueprints import Blueprint
from .jobs import ManagerProcess


class JobManager:
    tasks = ManagerProcess()
    password_key = None

    def __init__(self, app: Flask = None, password_key=None):
        if app:
            self.init_app(app)

        if password_key:
            self.set_password_key(password_key)

    def set_password_key(self, password_key):
        self.password_key = password_key

    def init_app(self, app: Flask):
        self.app = app
        self.app.extensions["job_manager"] = self
        app.extensions["job_manager"] = self
        if not self.password_key:
            password = os.environ.get("JOB_MANAGER_PASSWORD_KEY")
            if not password:
                if not app.secret_key:
                    raise ValueError("JOB_MANAGER_PASSWORD_KEY não definido!")
                password = app.secret_key
            self.set_password_key(password)
        self.blueprint = self.create_blueprint()
        self.add_urls(self.blueprint)
        app.register_blueprint(self.blueprint)
        return self

    def jobs(self, uuid=None):
        if not self.is_logged():
            return redirect(url_for("job_manager.login"))
        if request.method == "DELETE":
            processo = self.tasks.stop(uuid)
            component = self.render_job(uuid, processo)
            return component
        elif request.method == "POST":
            processo = self.tasks.restart(uuid)
            component = self.render_job(uuid, processo)
            return component
        data = {"jobs": tuple(self.tasks)}
        return render_template("/job_manager/index.html", data=data)

    def check_password(self, password):
        return password == self.password_key

    def set_logged(self):
        session["job_login"] = True

    def is_logged(self):
        return session.get("job_login", False)

    def set_logout(self):
        session.pop("job_login", None)

    def login(self):
        errors = {}
        if request.method == "POST":
            if self.check_password(request.form.get("acesso")):
                self.set_logged()
                return redirect(url_for("job_manager.jobs"))
            errors = {"acesso": "Login invalido!"}
        return render_template("/job_manager/login.html", errors=errors)

    def logout(self):
        self.set_logout()
        return redirect(url_for("job_manager.login"))

    def register_blueprint(self, blueprint: Blueprint):
        self.app.register_blueprint(blueprint)

    def add_urls(self, blueprint: Blueprint):
        blueprint.add_url_rule("/", "jobs", self.jobs)

    def create_blueprint(self, url_prefix="/jobs"):
        blueprint = Blueprint(
            "job_manager",
            __name__,
            url_prefix=url_prefix,
            template_folder="templates",
        )
        blueprint.add_url_rule(
            "/login/",
            "login",
            self.login,
            methods=["GET", "POST"],
        )
        blueprint.add_url_rule(
            "/",
            "jobs",
            self.jobs,
            methods=["GET"],
        )
        blueprint.add_url_rule(
            "/<uuid:uuid>/",
            "jobs-uuid",
            self.jobs,
            methods=["DELETE", "POST"],
        )
        blueprint.add_url_rule(
            "/logout/",
            "logout",
            self.logout,
            methods=["GET"],
        )
        return blueprint

    def render_job(self, uuid, job=None):
        component = """
        {% from 'job_manager/_component_job_macro.html' import ComponentJob %}
        {{
            ComponentJob(
                uuid=job.uuid,
                grupo=job.group,
                titulo=job.name,
                descricao=job.description,
                is_running=job.is_running,
                is_task=job.is_task,
                last_run=job.last_run,
                stoped=job.stoped
            )
        }}"""
        if job:
            component = render_template_string(
                component,
                job=job,
            )
        else:
            component = render_template_string(
                component,
                job=self.tasks.find_by_uuid(uuid),
            )

        return component



from abc import ABC, abstractproperty
import re


def clean_value(value, ignore=False):
    if ignore:
        return value
    return re.sub(r"\D", "", value)


class AbsValidatorType(ABC):
    ...


class AbstractValidator(AbsValidatorType):
    @abstractproperty
    def validate(self, doc: str) -> bool:
        ...

    @abstractproperty
    def mask(self, doc="") -> str:
        ...


class REGEX(AbsValidatorType):
    CLEAR = False

    @abstractproperty
    def REGEX(self) -> re.compile:
        ...

    @abstractproperty
    def MASK_MATCH(self) -> str:
        ...

    def validate(self, value) -> bool:
        value = clean_value(value, not self.CLEAR)
        value_mask = self.mask(value)
        if not value or not value_mask:
            return False
        if value == clean_value(value_mask, not self.CLEAR):
            return True

    def mask(self, value) -> str:
        if isinstance(self.REGEX, str):
            self.REGEX = re.compile(self.REGEX)
        return self.REGEX.sub(self.MASK_MATCH, value)


class REGEXMATCH(AbsValidatorType):
    CLEAR = False

    @abstractproperty
    def REGEX(self) -> re.compile:
        ...

    @abstractproperty
    def MASK_MATCH(self) -> str:
        ...

    def validate(self, value) -> bool:
        if isinstance(self.REGEX, str):
            self.REGEX = re.compile(self.REGEX)
        if self.REGEX.match(value):
            return True
        return False

    def mask(self, value) -> str:
        if isinstance(self.REGEX, str):
            self.REGEX = re.compile(self.REGEX)
        return self.REGEX.sub(self.MASK_MATCH, value)


class AbstractValidate(ABC):
    CLEAR = False

    @abstractproperty
    def MSG_RAISE(self):
        ...

    @abstractproperty
    def VALIDATOR(self):
        ...

    def validate(self, value):
        """Valida o documento.
        raise ValueError: caso o documento seja inválido"""
        if not self.VALIDATOR().validate(clean_value(value, not self.CLEAR)):
            raise ValueError(self.MSG_RAISE)
        return True

    def mask(self, value):
        return self.VALIDATOR().mask(clean_value(value, not self.CLEAR))



from multiprocessing import Process
from typing import Any
from uuid import uuid4
from loguru import logger
from datetime import datetime
import time
from typing import List


class Processo(Process):
    def __init__(
        self,
        name,
        target,
        group=None,
        daemon=True,
        uuid=None,
        description="",
        args=[],
        kwargs={},
    ):
        super().__init__(
            name=name,
            target=target,
            daemon=daemon,
            args=args,
            kwargs=kwargs,
        )
        self.uuid = None
        self.group = group
        self.target = target
        if uuid:
            self.uuid = uuid
        else:
            self.uuid = uuid4()
        self.is_task = True
        self.last_run = None
        self.stoped = None
        self.description = description
        self.args = args
        self.kwargs = kwargs

    @property
    def is_running(self):
        return self.is_alive()

    def __call__(self, *args: Any, **kwds: Any) -> Any:
        return self.target(*args, **kwds)

    def heranca(self):
        return {
            "name": self.name,
            "target": self.target,
            "group": self.group,
            "daemon": self.daemon,
            "uuid": self.uuid,
            "description": self.description,
            "args": self.args,
            "kwargs": self.kwargs,
        }

    def __repr__(self) -> str:
        return '<Processo name:"{}" uuid:"{}" group:"{}"/>'.format(
            self.name, self.uuid, self.group
        )

    def start(self) -> None:
        logger.info(f"{self} Iniciado!")
        self.last_run = datetime.utcnow()
        return super().start()

    def terminate(self) -> None:
        logger.info(f"{self} Terminado!")
        self.stoped = datetime.utcnow()
        return super().terminate()


class ManagerProcess:
    _storage = {}

    def _add(self, process: Process, group: str = None):
        """Adicionar um processo no setlist do group"""
        self._storage.setdefault(group, set()).add(process)

    def remove(self, process: Process, group: str = None):
        """Remove um processo do setlit do group"""
        process.is_alive() and process.terminate()
        self._storage.get(group, set()).remove(process)

    def restart(self, uuid):
        old = self.find_by_uuid(uuid)
        processo = self.new(
            **old.heranca(),
        )
        self.remove(old, old.group)
        processo.start()
        return processo

    def find_by_function(self, function):
        for processo in self._get_all():
            if processo.target == function:
                return processo

    def decorate(
        self,
        *args,
        name=None,
        group="decorador",
        description="",
        daemon=True,
        uuid=None,
        **kwargs,
    ):
        def wrapper(target):
            processo = self.new(
                name=name or target.__name__,
                target=target,
                group=group,
                description=description or target.__doc__ or "",
                daemon=daemon,
                uuid=uuid,
                *args,
                **kwargs,
            )
            return processo

        return wrapper

    def new(
        self,
        name,
        target,
        group="default",
        description="",
        daemon=True,
        uuid=None,
        args=[],
        kwargs={},
    ):
        processo = Processo(
            uuid=uuid,
            name=name,
            target=target,
            group=group,
            daemon=daemon,
            description=description,
            args=args,
            kwargs=kwargs,
        )
        self._add(processo, group)
        logger.success(f"{processo} Criado!")
        return processo

    def find_by_uuid(self, uuid):
        for processo in self._get_all():
            if processo.uuid == uuid:
                return processo

    def stop(self, uuid):
        processo = self.find_by_uuid(uuid)
        if processo:
            processo.terminate()
            return processo
        logger.warning(f"Processo com uuid '{uuid}' não encontrado!")
        return False

    @property
    def groups(self):
        return tuple(self._storage.keys())

    @classmethod
    def loop(cls):
        while True:
            time.sleep(1)

    def _get_all(self) -> List[Process]:
        for grupo in self._storage.values():
            for processo in grupo:
                yield processo

    def _get_all_by_group(self, group):
        for processo in self._storage.get(group, set()):
            yield processo

    def _get_process(self, group=None):
        if group:
            processos = self._get_all_by_group(group=group)
        else:
            processos = self._get_all()
        for processo in processos:
            yield processo

    def start(self, group=None):
        processos = self._get_process(group=group)
        for processo in processos:
            processo.start()

    def wait(self, group=None):
        processos = self._get_process(group=group)
        for processo in processos:
            processo.join()

    def __getitem__(self, name):
        return tuple(self._get_process(name))

    def __iter__(self):
        return self._get_process()



from validate_docbr import CPF, CNPJ
from .abs import AbstractValidate
from .types import CPFCNPJ, INCRICAOMUNICIPAL, EMAIL


class Cpf(AbstractValidate):
    CLEAR = True
    MSG_RAISE = "CPF inválido!"
    VALIDATOR = CPF


class Cnpj(AbstractValidate):
    CLEAR = True
    MSG_RAISE = "CNPJ inválido!"
    VALIDATOR = CNPJ


class CpfCnpj(AbstractValidate):
    CLEAR = True
    MSG_RAISE = "CPF ou CNPJ inválido!"
    VALIDATOR = CPFCNPJ


class InscricaoMunicipal(AbstractValidate):
    CLEAR = True
    MSG_RAISE = "Inscrição Municipal inválida!"
    VALIDATOR = INCRICAOMUNICIPAL


class Email(AbstractValidate):
    MSG_RAISE = "Email inválido!"
    VALIDATOR = EMAIL



from validate_docbr import CPF, CNPJ
import re
from .abs import AbstractValidator, REGEX, REGEXMATCH


class CPFCNPJ(AbstractValidator):
    def validate(self, doc) -> bool:
        doc = re.sub(r"\D", "", doc)
        if len(doc) == 11:
            return CPF().validate(doc)
        return CNPJ().validate(doc)

    def mask(self, doc="") -> str:
        doc = re.sub(r"\D", "", doc)
        if len(doc) == 11:
            return CPF().mask(doc)
        return CNPJ().mask(doc)


class INCRICAOMUNICIPAL(REGEX):
    CLEAR = True
    REGEX = r"(\d{3})(\d{3})(\d)"
    MASK_MATCH = r"\1.\2-\3"


class EMAIL(REGEXMATCH):
    REGEX = r"([\w\W]+)(@)([\w\W]+)(.)([\w\W]+)"
    MASK_MATCH = r"\1\2\3\4\5"

    def validate(self, value) -> bool:
        return super().validate(value) and "." in value and "@" in value



from pydantic import BaseModel


class CredencialEmail(BaseModel):
    email: str
    senha: str


class CredencialUsername(BaseModel):
    username: str
    senha: str


class CredencialCnpj(BaseModel):
    cnpj: str
    senha: str


class CredencialCpf(BaseModel):
    cpf: str
    senha: str


class CredencialLogin(BaseModel):
    login: str
    senha: str



import re
from typing import Optional
from requests import get
from pydantic import BaseModel


class CepError(Exception):
    ...


class CepNotFound(CepError):
    ...


def clear_cep(cep: str):
    return re.sub(r"\D", "", str(cep))


def serializer_cep(cep: str):
    if len(cep) >= 5:
        return f"{cep[:5]}{cep[5:]:0<3}"
    return cep


def mask_cep(cep: str):
    cep = clear_cep(cep)
    if len(cep) >= 5:
        return f"{cep[:5]}-{cep[5:]:0<3}"
    return cep


class CepCordenadas(BaseModel):
    latitude: Optional[float] = None
    longitude: Optional[float] = None


class CepLocation(BaseModel):
    type: Optional[str] = None
    coordinates: Optional[CepCordenadas] = None


class CepConsultaV2(BaseModel):
    cep: Optional[str] = None
    state: Optional[str] = None
    city: Optional[str] = None
    neighborhood: Optional[str] = None
    street: Optional[str] = None
    service: Optional[str] = None
    location: Optional[CepLocation] = None


class CepConsultaV1(BaseModel):
    cep: Optional[str] = None
    state: Optional[str] = None
    city: Optional[str] = None
    neighborhood: Optional[str] = None
    street: Optional[str] = None
    service: Optional[str] = None


class Cep:
    CepNotFound = CepNotFound
    CepError = CepError

    def __init__(self, version):
        self.version = version

    def _get_url(self, cep: str):
        return f"https://brasilapi.com.br/api/cep/v{self.version}/{cep}"

    def _response(self, response, cep: str):
        if response.status_code == 200:
            return response.json()
        else:
            raise CepNotFound(f"CEP {mask_cep(cep)} não encontrado")

    def _serializer(self, data: dict):
        if self.version == 2:
            return CepConsultaV2(**data)
        return CepConsultaV1(**data)

    def get(self, cep: str):
        cep = serializer_cep(cep)
        url = self._get_url(cep)
        response = self._response(get(url), cep)
        return self._serializer(response)



from dynaconf import Dynaconf
from loguru import logger
from pydantic import BaseModel
from enum import Enum
import os


class EnvEnum(Enum):
    DEV = "DEV"
    PRD = "PRD"


class TypeEnv(BaseModel):
    ENV: EnvEnum
    DEBUG: bool = False

    @classmethod
    def create_config(cls, settings, env):
        SETTINGS_DICT = settings.as_dict()
        ENV_ROOT = dict(os.environ)
        DEFAULT_SETTINGS = SETTINGS_DICT.pop("DEFAULT", {})
        ENV_SETTINGS = SETTINGS_DICT.pop(env, {})

        obj = cls(
            **{
                **DEFAULT_SETTINGS,
                **ENV_SETTINGS,
                **SETTINGS_DICT,
                **ENV_ROOT,
                "ENV": env,
            }
        )
        obj._settings = settings
        return obj

    @classmethod
    def load_env(cls, default, env, settings_files=[], prefix=""):
        return cls.create_config(
            Dynaconf(
                environments=True,
                envvar_prefix=prefix,
                default_env=default,
                env=env,
                settings_files=settings_files,
            ),
            env=env,
        )

    def show(self):
        text_env = f"VARIAVEIS DE AMBIENTE: {self.ENV.value}\n"
        envs = [f"{key} = {value}" for key, value in dict(self).items()]
        text_env += "\n".join(envs)
        logger.warning(text_env)



from setuptools import setup


import os


def listar_subpastas(diretorio):
    subpastas = [diretorio]
    for nome in os.listdir(diretorio):
        if nome == "__pycache__":
            continue
        caminho = os.path.join(diretorio, nome)
        if os.path.isdir(caminho):
            subpastas.append(caminho)
            subpastas.extend(listar_subpastas(caminho))
    return subpastas


with open("README.md", "r") as fh:
    readme = fh.read()

setup(
    name="feiticeiro_tec",
    version="0.3.0",
    url="https://github.com/feiticeiro-tec/feiticeiro-tec",
    license="BSD3",
    author="Silvio Henrique Cruz Da Silva",
    long_description=readme,
    long_description_content_type="text/markdown",
    author_email="silviohenriquecruzdasilva@gmail.com",
    keywords="python, feiticeiro_tec, utilidades, feiticeiro, tec",
    description=("pacote de utilidades para projetos python"),
    packages=listar_subpastas("feiticeiro_tec"),
    package_data={
        "feiticeiro_tec": [
            "flask/job_manager/templates/job_manager/*",
        ],
    },
    install_requires=[
        "loguru",
        "pydantic",
    ],
    extras_require={
        "dynaconf": ["dynaconf"],
        "validate": ["validate_docbr"],
        "request": ["requests"],
    },
)




from flask import Flask, request
import gzip
from functools import wraps


class Compress:
    def __init__(self, app: Flask):
        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        self.app = app
        self.app.after_request(self.after_request)

    @classmethod
    def after_request(cls, response):
        if request.headers.get("Accept-Encoding", "") != "gzip":
            return response
        if (
            response.status_code < 200
            or response.status_code >= 300
            or "Content-Encoding" in response.headers
        ):
            return response
        response.data = gzip.compress(response.data)
        response.headers["Content-Encoding"] = "gzip"
        response.headers["Content-Length"] = len(response.data)
        return response

    def compress_decorator(self, function):
        @wraps(function)
        def decorated_function(*args, **kwargs):
            response = function(*args, **kwargs)
            return self.after_request(response)

        return decorated_function
